#!/usr/bin/env python
import sys
import json
import ast
import subprocess
import traceback

from charmhelpers.core import unitdata
from charmhelpers.core.hookenv import (
    log, relation_ids, relation_get,
    relation_set, Hooks, UnregisteredHookError)

from charmhelpers.core.host import (service_restart)
from cinder_contexts import (NedgeSubordinateContext)
from charmhelpers.contrib.openstack.utils import (get_os_codename_package)
from utils import (install_iscsi_driver)

hooks = Hooks()
hook_data = unitdata.HookData()
db = unitdata.kv()


@hooks.hook('install')
def install():

    try:
        log("Cinder-Nedge install:")
        os_release = get_os_codename_package('cinder-common', fatal=False)
        log("Installing driver for OS release: {}".format(os_release))
        install_iscsi_driver(os_release)

    except subprocess.CalledProcessError as cpe:
        log('Failed!\nMessage:\n{0}\nTrace:\n{1}\nOutput:\n{2}'
            .format(cpe.message, traceback.format_exc(), cpe.output))
        sys.exit(1)
    except Exception as e:
        log('Installation failed!\nMessage:{0}\nTrace:{1}'
            .format(e.message, traceback.format_exc()))
        sys.exit(1)


@hooks.hook('cinder-nedge-relation-changed')
def on_gateway_data_changed():
    context = relation_get('cinder-context')
    log("Recived data from GW")
    if context:
        context = ast.literal_eval(context)
        log("[CINDER-NEDGE] context received from CINDER GW: {}".format(
            context))
        db.set('cinder-context', context)
        for relid in relation_ids('storage-backend'):
            on_storage_backend_joined(relid)
    else:
        log("No cinder-context recived!")


@hooks.hook('storage-backend-relation-joined')
def on_storage_backend_joined(relid=None):
    context = db.get('cinder-context', record=True)
    if context:
        log("Cinder context: {}".format(context))
        log("SEND cinder-context to storage-backend")
        relation_set(relation_id=relid,
                     backend_name='nedge',
                     subordinate_configuration=json.dumps(
                         NedgeSubordinateContext()(context)),
                     stateless=False, )
        # restart cinder service
        service_restart('cinder-volume')
    else:
        log("[CN] backend joined. No context to send")


if __name__ == '__main__':
    try:
        with hook_data():
            hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        log('Unknown hook {} - skipping.'.format(e))
